#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for a cloud service."""


import urlparse
import threading

import libpal.palcmd as palcmd
from iobase.xmldata import (XmlConfNode, ANY_CHILD_NAME,)
from pal_info import (ReqTypes, Results)

try:
    from landiskcloud.proxy import (PROXY_NAME, HTTPS_PROTOCOL, ProxyInfo,)
    from landiskaccess.fletsazukeruaccess import (FletsAzukeruAccess,)
    from landiskcloud.info.fletsazukeru import (FletsAzukeruConnectionInfo,
                                                FletsAzukeruTargetInfo)
    from pkgaddon.flets_azukeru_config import (FLETS_AZUKERU_CONF_PATH,)
except:
    pass



URL_XML_TMPL = "<url>%s</url>"

SUCCESS_RESULT = "Success"
FAILED_RESULT = "Failed"
TIMEOUT_RESULT = "TimeOut"
LOGIN_FAILED_RESULT = "LoginFailed"

CHECK_TIMEOUT_SEC = 20
RESULT_KEY = "result"
FLETS_AZUKERU_ROOT_PATH = "/"

RESULT_NAME = "result"

FLETSAZUKERU_NAME = "fletsazukeru"


def _exec_check(client, result_dict):
    """Thread main for checking to access to the BOX server."""

    try:
        client.stat_directory(FLETS_AZUKERU_ROOT_PATH)
        result_dict[RESULT_KEY] = SUCCESS_RESULT

    except LoginError:
        result_dict[RESULT_KEY] = LOGIN_FAILED_RESULT

    except:
        result_dict[RESULT_KEY] = FAILED_RESULT


class PalCloud(palcmd.PalCmd):
    """PAL Cloud command implementation class."""

    def __get_handler(self, entry):
        """Handle get request."""

        target_node = entry.get().get_node(ANY_CHILD_NAME)
        target_node.append_child(XmlConfNode(URL_XML_TMPL %
                                                "http://dummy/dummy"))
        entry.set_result(Results.OK)
        return entry

    def __check_handler(self, entry):
        """Handle check request."""

        result = SUCCESS_RESULT
        try:
            target_node = entry.get().get_node(ANY_CHILD_NAME)
            if target_node.get_node_name() == FLETSAZUKERU_NAME:
                connection_info = FletsAzukeruConnectionInfo(target_node)

                try:
                    proxy_info = ProxyInfo(entry.get().get_node(PROXY_NAME))
                    proxy = proxy_info.get_proxy_url(HTTPS_PROTOCOL)
                except:
                    proxy = None

                client = FletsAzukeruAccess(
                        connection_info.clientid,
                        connection_info.password,
                        connection_info.host_name,
                        connection_info.company_id,
                        connection_info.service_id,
                        FletsAzukeruTargetInfo.MY_FOLDER,
                        terminal_id="",
                        terminal_key=connection_info.terminalkey,
                        using_image_login=connection_info.using_image_login,
                        conf_path=FLETS_AZUKERU_CONF_PATH,
                        proxy=proxy)

            result_dict = {}
            check_thread = threading.Thread(target=_exec_check,
                                            args=(client, result_dict))
            check_thread.daemon = True
            check_thread.start()
            check_thread.join(CHECK_TIMEOUT_SEC)

            if check_thread.is_alive():
                result = TIMEOUT_RESULT
            else:
                result = result_dict[RESULT_KEY]
        except:
            self.exception("Check Failed!!")
            result = FAILED_RESULT

        if result == SUCCESS_RESULT:
            entry.set_result(Results.OK)
        else:
            entry.set_result(Results.CHECK_ERROR)

        entry.get().append_child(RESULT_NAME, result)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.GET, self.__get_handler)
        self.set_handler(ReqTypes.CHECK, self.__check_handler)

#
# Main
#

if __name__ == "__main__":
    PalCloud(option=palcmd.PalCmdOption()).handle_request()
